{*************************************************************}
{              KDatabase Export pack version 1.00             }
{               Copyright  2000 Korzh company                }
{                http://www.korzh.com/delphi/                 }
{                  mailto:delphi@korzh.com                    }
{-------------------------------------------------------------}
{                  TKDataSetToHTML component                  }
{                  last updated: Mar-16-2000                  }
{*************************************************************}
unit kds2html;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DB;

type
  TKHTMLFormatCellEvent = procedure (Sender: TObject; CellRow, CellColumn: Integer; FieldName : string;
                                    var CustomAttrs, CellData: string) of object;

  TKDataSetToHTML = class(TComponent)
  private
    FDataSet : TDataSet;
    FHeader : TStrings;
    FFooter : TStrings;
    FOnFormatCell : TKHTMLFormatCellEvent;
    procedure SetHeader(Value : TStrings);
    procedure SetFooter(Value : TStrings);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation);override;
  public
    constructor Create(AOwner : TComponent);override;
    destructor Destroy; override;
    procedure Transfer(const FileName : string);
  published
    property DataSet : TDataSet read FDataSet write FDataSet;
    property Header : TStrings read FHeader write SetHeader;
    property Footer : TStrings read FFooter write SetFooter;
    property OnFormatCell : TKHTMLFormatCellEvent read FOnFormatCell write FOnFormatCell;
  end;

implementation

constructor TKDataSetToHTML.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  FDataSet := nil;
  FHeader := TStringList.Create;
  FFooter := TStringList.Create;
  FOnFormatCell := nil;
end;

destructor TKDataSetToHTML.Destroy;
begin
  FFooter.Free;
  FHeader.Free;
  inherited;
end;

procedure TKDataSetToHTML.SetHeader(Value : TStrings);
begin
  FHeader.Assign(Value);
end;

procedure TKDataSetToHTML.SetFooter(Value : TStrings);
begin
  FFooter.Assign(Value);
end;

procedure TKDataSetToHTML.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FDataSet) then
    FDataSet := nil;
end;

procedure TKDataSetToHTML.Transfer(const FileName : string);
var
  HTML : TStrings;
  i, RowNum, ColNum : integer;
  CustomAttrs : string;
  S : string;
begin
  HTML := TStringList.Create;
  try
    if FHeader.Count > 0 then
      HTML.Assign(FHeader)
    else begin
      HTML.Add('<html>');
      HTML.Add('<head> <title> DataSet </title> </head>');
      HTML.Add('<body>');
    end;
    HTML.Add('<table border = "1" rows = "' + IntToStr(FDataSet.RecordCount) +
              '" bordercolor="#000000" cellspacing="0" cellpadding="2">');
    with FDataSet do
    begin
      //filling table header
      HTML.Add('<tr>');  ColNum := 1;
      for i := 0 to FieldCount - 1 do
      begin
        if Fields[i].Visible then
        begin
          CustomAttrs := '';
          S := Fields[i].DisplayLabel;
          if Assigned(FOnFormatCell) then FOnFormatCell(Self, 1, ColNum,
                                                         Fields[i].FieldName, CustomAttrs, S);
          HTML.Add('<th ' + CustomAttrs + ' >' + S + ' </th>');
          inc(ColNum)
        end;
      end;
      HTML.Add('</tr>');

      First;RowNum := 2;
      while not EOF do
      begin
        HTML.Add('<tr>');ColNum := 1;
        for i := 0 to FieldCount - 1 do
        begin
          if Fields[i].Visible then
          begin
            CustomAttrs := '';
            if Fields[i].DataType = ftCurrency then
            begin
              S := FloatToStrF(Fields[i].AsCurrency, ffCurrency, 20, 2)
            end
            else
              S := Fields[i].AsString;
            if S = '' then S := '&nbsp';

            if Assigned(FOnFormatCell) then FOnFormatCell(Self, RowNum, ColNum,
                                                           Fields[i].FieldName, CustomAttrs, S);
            HTML.Add('<td nowrap ' + CustomAttrs + ' >' + S + ' </td>');
            inc(ColNum)
          end;  
        end;
        HTML.Add('</tr>');
        FDataSet.Next;inc(RowNum);
      end;
    end;  
    HTML.Add('</table>');
    if FFooter.Count > 0 then
      HTML.AddStrings(FFooter)
    else begin
      HTML.Add('</body>');
      HTML.Add('</html>');
    end;
    HTML.SaveToFile(FileName);
  finally
    HTML.Free;
  end;
end;

end.
